#include <boost/version.hpp>
#include "TransformPy.h"

using namespace boost::python;

    void exportTransform()
    {
      // Disable autogeneration of C++ signatures (Boost 1.34.0 and higher)
      // for Epydoc which stumbles over indentation in the automatically generated strings.
      boost::python::docstring_options no_autogen(true,false);

      class_<Transform>(
        "Transform",
        "A class defining a general eometric Transformation.",
        init<>()
      )
      .def(init<const Transform &>())
      .def(
        "translate",
        &Transform::translate,
        ( arg("x"), arg("y"), arg("z") ),
        "Adds a translation by a specified amount to the translation\n"
        "@type x: L{double}\n"
        "@rtype: L{Transform}\n"
      )
      .def(
        "addTranslation",
        &Transform::addTranslation,
        ( arg("x"), arg("y"), arg("z") ),
        "Adds a translation by a specified amount to the transformation\n"
        "@type x: L{double}\n"
        "@type y: L{double}\n"
        "@type z: L{double}\n"
      )
      .def(
        "rotateX",
        &Transform::rotateX,
        ( arg("phi")),
        "Adds a rotation aroung the x-axis. N.B. argument in radians \n"
        "@type x: L{phi}\n"
        "@rtype: L{Transform}\n"
      )
      .def(
        "rotateY",
        &Transform::rotateY,
        ( arg("phi")),
        "Adds a rotation aroung the x-axis. N.B. argument in radians \n"
        "@type x: L{phi}\n"
        "@rtype: L{Transform}\n"
      )
      .def(
        "rotateZ",
        &Transform::rotateZ,
        ( arg("phi")),
        "Adds a rotation aroung the z-axis. N.B. argument in radians \n"
        "@type x: L{phi}\n"
        "@rtype: L{Transform}\n"
      )
      .def(
        "scale",
        &Transform::scale,
        ( arg("scale")),
        "return scaled transform \n"
        "@type scale: L{float}\n"
        "@rtype: L{Transform}\n"
      )
      .def(
        "addRotationX",
        &Transform::addRotationX,
        ( arg("phi")),
        "Adds a rotation aroung the z-axis. N.B. argument in radians \n"
        "@type phi: L{float}\n"
      )
      .def(
        "addRotationY",
        &Transform::addRotationY,
        ( arg("phi")),
        "Adds a rotation aroung the z-axis. N.B. argument in radians \n"
        "@type phi: L{float}\n"
      )
      .def(
        "addRotationZ",
        &Transform::addRotationZ,
        ( arg("phi")),
        "Adds a rotation aroung the z-axis. N.B. argument in radians \n"
        "@type phi: L{float}\n"
      )
      .def(
        "addScale",
        &Transform::addScale,
        ( arg("scale")),
        "Adds an isotropic scaling. \n"
        "@type scale: L{float}\n"
      )
      .def(
		  "transform",
		  &Transform::transform,
		  ( arg("v") ),
		  "transform a given point\n"
		  "@type v: L{Vector3}\n"
		  "@rtype: L{Vector3}\n"
     )
     .def(
		  "transformInverse",
		  &Transform::invtransform,
		  ( arg("v") ),
		  "inverse transform a given point\n"
		  "@type v: L{Vector3}\n"
		  "@rtype: L{Vector3}\n"
     )
	 .def(
        "getScale",
        &Transform::getScale,
        "Returns the isotropic scaling component of the transformation.\n"
        "@rtype: L{float}\n"
      )
     .def(self_ns::str(self))
     ;
    }
