#include <boost/version.hpp>
#include "TransformableBox3DPy.h"

using namespace boost::python;

    void exportTransformableBoxWithPlanes3D()
    {
      // Disable autogeneration of C++ signatures (Boost 1.34.0 and higher)
      // for Epydoc which stumbles over indentation in the automatically generated strings.
      boost::python::docstring_options no_autogen(true,false);

      class_<TransformableBoxWithPlanes3D, bases<BoxWithPlanes3D,AVolume3D> >(
        "TransformableBoxWithPlanes3D",
        "A class defining a rectangular volume in 3D optionally bounded by planes.",
        init<>()
      )
      .def(init<const TransformableBoxWithPlanes3D &>())
      .def(
        init<Vector3,Vector3>(
          ( arg("minPoint"), arg("maxPoint") ),
          "Constructs a box with the specified corner points.\n"
          "@type minPoint: L{Vector3}\n"
          "@kwarg minPoint: Coordinate of bottom-left-front corner of the box\n"
          "@type maxPoint: L{Vector3}\n"
          "@kwarg maxPoint: Coordinate of upper-right-back corner of the box\n"
        )
      )
      .def(
        "addPlane",
        &TransformableBoxWithPlanes3D::addPlane,
        ( arg("plane") ),
        "Adds a plane to the box for fitting particles.\n"
        "@type plane: L{Plane}\n"
        "@kwarg plane: the plane to add to the box\n"
        "@rtype: void\n"
      )
      .def(
        "addTransform",
        &TransformableBoxWithPlanes3D::addTransformation,
        ( arg("transform") ),
        "Add a transformation to the Box, i.e. the current\n"
		"transformation matrix is multiplied by the supplied transformation.\n"
        "@type transform: L{Transform}\n"
        "@kwarg transform: the new transformation\n"
        "@rtype: void\n"
      )
      .def(
        "transform",
        &TransformableBoxWithPlanes3D::transform,
        ( arg("transform") ),
        "Generate a new, transformed Box. The current Box is unchanged\n"
        "@type transform: L{Transform}\n"
        "@kwarg transform: the new transformation\n"
        "@rtype: void\n"
      )
      .def(self_ns::str(self))
      ;
//      boost::python::implicitly_convertible<BoxWithPlanes3D, AVolume2D>();
    }