#include <boost/version.hpp>
#include "TransformableMeshVolPy.h"

using namespace boost::python;

void exportTransformableMeshVolume()
{
  // Disable autogeneration of C++ signatures (Boost 1.34.0 and higher)
  // for Epydoc which stumbles over indentation in the automatically generated strings.
  boost::python::docstring_options no_autogen(true,false);

  class_<TransformableMeshVolume, bases<MeshVolume,AVolume3D> >(
	"TransformableMeshVolume",
	"A class defining a transformable volume in 3D bounded by a set of triangles.",
	init<>()
  )
  .def(init<const TransformableMeshVolume &>())
  .def(
        init<const TriPatchSet& >(
          ( arg("Mesh")),
          "Constructs a volume from a supplied set of triangles.\n"
          "@type Mesh: L{TriPatchSet}\n"
          "@kwarg Mesh: The set of triangles\n"
        )
      )
  .def(
	"addTransform",
	&TransformableMeshVolume::addTransformation,
	( arg("transform") ),
	"Add a transformation, i.e. the volume \n"
	"is transformed by the supplied transformation.\n"
	"@type transform: L{Transform}\n"
	"@kwarg transform: the transformation\n"
	"@rtype: void\n"
  )
  .def(
	"transform",
	&TransformableMeshVolume::transform,
	( arg("transform") ),
	"Generate a new, transformed volume. The current volumes is unchanged\n"
	"@type transform: L{Transform}\n"
	"@kwarg transform: the new transformation\n"
	"@rtype: void\n"
  )
  .def(self_ns::str(self))
  ;
}