# include "AGeometricObject.h"
#include <iostream>

/*!
    Default constructor.
*/
AGeometricObject::AGeometricObject() 
{}

/*!
    calculate the derivatives of the distance function
    at a given location. 
    This is a default implementation approximating the 
    derivative from finite differences. Should be reimplemented
    in derived classes for better performance & accuracy.
     
    \param x the position at which the derivatives are calculated 
*/
std::array<double,3> AGeometricObject::getDistDeriv(const Vector3& x) const
{
    std::array<double,3> res;
    double delta=1e-8;
    
    double d0=this->getDist(x);
    double dx=this->getDist(x+Vector3(delta,0.0,0.0));
    double dy=this->getDist(x+Vector3(0.0,delta,0.0));
    double dz=this->getDist(x+Vector3(0.0,0.0,delta));
    
    res[0]=(dx-d0)/delta;
    res[1]=(dy-d0)/delta;
    res[2]=(dz-d0)/delta;
    
    return res;
}

void AGeometricObject::print() const
{
    std::cout << "AGeometricObject" << std::endl;
}
