#ifndef __TRANSFORM_H
#define __TRANSFORM_H

// project includes
#include "util/vector3.h"
#include "util/matrix4.h"

/*!
    class for geometric transformations
*/
class Transform
{
    private:
        Matrix4 m_mat;
        Matrix4 m_inv_mat; // matrix for the inverse transform - saves having to invert m_mat
		double m_scale; // track scaling explicitly to avoid calculating eigenvalues in getScale()
    
    public:
        Transform();
        Transform(const Transform&);
    
        virtual ~Transform();

        // apply to 3-vector
        Vector3 transform(const Vector3&) const;
		Vector3 invtransform(const Vector3&) const;
    
        // apply to the _normal_ of an object 
        Vector3 transform_n(const Vector3&) const;
		Vector3 invtransform_n(const Vector3&) const;
    
        // add geometric transform
        void addTranslation(double, double, double);
        void addRotationX(double);
        void addRotationY(double);
        void addRotationZ(double);
		void addScale(double);
		void addTransform(const Transform&);
    
        Transform translate(double,double,double) const;
        Transform rotateX(double) const;
        Transform rotateY(double) const;
		Transform rotateZ(double) const;
		Transform scale(double) const;
		
		double getScale() const;
	
        friend std::ostream& operator << (std::ostream&,const Transform&);        
}; 
    
#endif // __TRANSFORM_H
