#include "mat4.h"
#include <cmath>

/*!
	standard constructor. Initializes all elements to 0
*/
mat4::mat4() : m_mat(0.0,16)
{}
				
/*!
	solve system of equations by Gauss-Seidel elimination, 
	destroying the matrix in the process

	\param rhs right hand side vector
	\returns solution. 
*/
mat4::solution mat4::solve_destructive(std::valarray<double> rhs,double tol)
{
	mat4::solution res;
	res.data=std::valarray<double>(0.0,4);
	res.valid=false;
	
	double max;
	double temp;

	unsigned int m;


	for(unsigned int k=0; k<3; k++) {// base row of matrix
		// search of line with max element
		max = fabs(m_mat[k*4 + k]);
		m = k;
		for(unsigned int i=k+1; i<4; i++) {
			if (max < abs(m_mat[i*4 + k])){
				max = m_mat[i*4 + k];
				m = i;
			}
		}
    
		// permutation of base line (index k) and max element line(index m)
		if(m != k){
			for(unsigned int i=k; i<4; i++){ // TODO rewrite with slices
				temp = m_mat[k*4 + i];
				m_mat[k*4 + i] = m_mat[m*4 + i];
				m_mat[m*4 + i] = temp;
			}
			temp = rhs[k];
			rhs[k] = rhs[m];
			rhs[m] = temp;
		}

		//std::cout << "k,k: " << fabs(m_mat[k*4 + k]) << std::endl;
		if( fabs(m_mat[k*4 + k]) < tol) {
			res.valid=false;
			return res;
		} 

		// triangulation of matrix with coefficients
		for(unsigned int j=(k+1); j<4; j++){ // current row of matrix
			temp = -1.0*m_mat[j*4+k] / m_mat[k*4+k];
			for(unsigned int i=k; i<4; i++) { // TODO rewrite with slices
				m_mat[j*4+i] = m_mat[j*4+i] + temp*m_mat[k*4+i];
			}
			rhs[j] = rhs[j] + temp*rhs[k]; // free member recalculation
		}
	}

	for(int k=3; k>=0; k--)	{
		res.data[k] = rhs[k];
		for(unsigned int i=(k+1); i<4; i++)
		{
			res.data[k] -= (m_mat[k*4 + i]*res.data[i]);
		}
		res.data[k] = res.data[k] / m_mat[k*4 + k];
	}
	
	res.valid=true;
	return res;
}

/*!
  matrix-vector multiplication
*/
std::valarray<double> mat4::operator* (const std::valarray<double>& rhs) const
{
	std::valarray<double> res(0.0,4);

	for(unsigned int i=0;i<4;i++){
		for(unsigned int j=0;j<4;j++){
			res[i]+=m_mat[i*4+j]*rhs[j];
		}
	}

	return res;
}
		
/*!
	output operator 
*/		
std::ostream& operator << (std::ostream& ost, const mat4& m)
{
	for(unsigned int i=0;i<4;i++){
		for(unsigned int j=0;j<4;j++){
			ost << m(i,j) << " , ";
		}
		ost << std::endl;
	}	
	
	return ost;
} 
