#include "TransformableMeshVolume.h"

/*!
	default contructor - construct empty mesh volume
*/
TransformableMeshVolume::TransformableMeshVolume() : MeshVolume()
{}

/*!
  Constructor to set up a TransformableMeshVolume from a triangle patch set

  \param tp the triangle set
*/
TransformableMeshVolume::TransformableMeshVolume(const TriPatchSet& tp) : MeshVolume(tp)
{
}

/*!
	Copy contructor
*/
TransformableMeshVolume::TransformableMeshVolume(const TransformableMeshVolume& T) : MeshVolume(T)
{
}

/*!
	Copy contructor with additional flag if acceleration grid is copied or not. 
*/
TransformableMeshVolume::TransformableMeshVolume(const TransformableMeshVolume& T, bool copy_grid) : MeshVolume(T,copy_grid)
{
}

/*!
	destructor
*/
TransformableMeshVolume::~TransformableMeshVolume()
{}


/*!
	Add a transformation to the volume, i.e. the current
	volume is transformed by the supplied transformation.
	
	\param T the new transformation
*/
void TransformableMeshVolume::addTransformation(const Transform& T)
{
	// transform mesh
	m_mesh.addTransformation(T);
	// update bounding box
	m_MinPoint=m_mesh.getBoundingBox().first;
	m_MaxPoint=m_mesh.getBoundingBox().second;
	m_DistPoint=m_MinPoint-Vector3(1.0,1.0,1.0);
	// rebuild acceleration grid
	setup_accel_grid();
}

/*!
	Generate a new, transformed volume. The original volume remains unchanged.

	\param T the transform
*/
TransformableMeshVolume TransformableMeshVolume::transform(const Transform& T) const
{
	TransformableMeshVolume res(*this,false);
		
	res.addTransformation(T);
	res.setup_accel_grid();
	
	return res;
}


ostream& operator<< (ostream& ost, const TransformableMeshVolume& TMV)
{
	ost << "TransformableMeshVolume" << std::endl;
	ost << TMV.m_mesh.size() << " triangles" << std::endl;
	ost << "bounding box: " << TMV.m_MinPoint << " - " << TMV.m_MaxPoint << std::endl;
	ost << TMV.m_mesh << std::endl;
	
	return ost;
}
